;===============================================================================
FUNCTION Disp_filt, Img, gradient=gradient
    ; Purpose: Apply image processing to images before display
    ;Parameters:
    ;   Img           INPUT intput array
    ;   Keywords
    ;     gradient     Input  Flag to control if a gradient ( edge) image is returned

    ;   Return:    a filtered image

    IF(not keyword_set(gradient))THEN gradient = !false

    ; get image dimensions
    ImgSize = size(Img,/dim)

    ; apply multi-order median filtering
    order = 1
    FiltImg = median(Img, 3)
    repeat begin
        FiltImg = median(TEMPORARY(FiltImg), 3)
        order = order + 1
    endrep until(order eq 4)

    if(gradient)then begin
        FiltImg = sobel(FiltImg)
        ; Clean-up edges of image
        FiltImg[0:1,*] = 0
        FiltImg[ImgSize[0]-2:*,*] = 0
        FiltImg[*,0:1] = 0
        FiltImg[*,ImgSize[1]-2:*] = 0
    endif

    RETURN, FiltImg
END

;===============================================================================
Function DispFrame, pStateInfo, pBoundary=pBoundary, frameData = frameData, dispwindow = dispwindow, image_mean=image_mean, return_image=return_image


;   This routine displays a set of ROI particles in the Draw window
;   Input
;     pStateInfo:   main structure holding all info

; Keyword
;   pBoundary     Input  pointer array of perimeter pixels, if present display with perimeter
;   frameData     Input  structure of frame data, if not present, use currently stored frame

; results
;   the frame is drawn to the display taking into acount all GUI settings

;return
;   the bounds of each particle are returned in the bounds array


if(keyword_set(dispwindow)) then begin
    wset, dispwindow
endif else begin
    wset, (*pStateInfo).MViewID
endelse

ERASE

if(not keyword_set(frameData))then begin
    ; ROIArray is an array of pointers for each particle in the frame
    ROIArray = *(*(*pStateInfo).pFrameData).pFrame

endif else begin
    ROIArray = *frameData.pFrame

endelse


; How many ROIs?
n_imgs = n_elements(  ROIArray )

; get zoom factor from gui
;widget_control,(*pStateInfo).ZoomID,get_value = zoom
zoom = (*pStateInfo).zoom

; get aspect ratio from window
widget_control,(*pStateInfo).AspectID,get_value = YXAspect

mosaic = (*pStateInfo).settings.mosaic

;get scroll size in x
geomInfo = widget_info((*pStateInfo).MainView,/geometry)

; get scale values
WIDGET_CONTROL, (*pStateInfo).ScaleMinID,get_value = scaleMin
WIDGET_CONTROL, (*pStateInfo).ScaleMaxID,get_value = scaleMax

oX = 0
oY = 0
maxdy = 0
bounds = intarr(n_imgs,4)

FOR i=0, n_imgs - 1 DO begin

    ;determine size of image including zoom factor
    dx = ROIArray[i].ImageHeader.ex - ROIArray[i].ImageHeader.sx + 1
    dy = ROIArray[i].ImageHeader.ey - ROIArray[i].ImageHeader.sy + 1
    dx = round(zoom * YXAspect * dx)
    dy = round(zoom * dy)

    if( Mosaic )then begin
        oX = zoom * YXAspect * ROIArray[i].ImageHeader.sx
        oY = zoom * ((*pStateInfo).FileHead.DY - 1 - ROIArray[i].ImageHeader.ey)
        x = [0,round(zoom * ((*pStateInfo).FileHead.DX - 1)),round(zoom * ((*pStateInfo).FileHead.DX - 1)),0,0]
        y = [0,0,round(zoom * ((*pStateInfo).FileHead.DY - 1)),round(zoom * ((*pStateInfo).FileHead.DY - 1)),0]
        plots,x,y,color=200,/device
    endif

    maxdy = maxdy > dy

    if(keyword_set(dispwindow)) then begin
        wset, dispwindow
    endif else begin
        wset, (*pStateInfo).MViewID
    endelse

    ; TAF 2014-03-19
    ; this is where the normal images are displayed in the CPI View GUI
    ; putting in a temporary stop to get the background means.

    ; modify where this is called in left_event.pro under IMGNUMSLIDER
    ; to grab the raw images and means from the whole file

    ; also note that disabling Autoscale and Background subtraction is necessary to get raw data

    if(!enable_image_diagnostic) then begin
        img = *ROIArray[i].pROI

        if((size(img))[1] gt !diagnostics_min_size) then begin
            image_mean = mean(img)

            return_image = img
        endif else begin
            image_mean = -1

            return_image = bytarr(2,2)
        endelse
    endif




    if((*pStateInfo).DispMode gt 0)then begin
        ; get settings from gui
        gradient = ((*pStateInfo).DispMode and 2)/2
        img = Disp_filt(*ROIArray[i].pROI, gradient=gradient)
        if(keyword_set(pBoundary))then if(ptr_valid(pBoundary[i]))then img[*(pBoundary[i])] = 255

        ; display the image
        ;tv,congrid(bytscl(img,min = scaleMin,max = scaleMax,top=255,/interp),dx,dy),oX,oY ; TAF 2014-03-26 /interp is not a keyword for bytscl
        tv, congrid(bytscl(img, min = scaleMin,max = scaleMax, top=255), dx, dy, /interp), oX, oY ; TAF 2014-03-26 /interp is not a keyword for bytscl

    endif else begin

        img = *ROIArray[i].pROI
        if(keyword_set(pBoundary))then if(ptr_valid(pBoundary[i]))then img[*(pBoundary[i])] = 255

        ; display the image
        tv, congrid(bytscl(img, min = scaleMin, max = scaleMax, top=255), dx, dy, /interp), oX, oY

    endelse
    ;stop;   here for out_lib.pro ;dx and dy =31,33 teh cut square of the image.

    ; store bounding box coords for checking is mouse is in box
    bounds[i,*] = [oX,oY,oX+dx-1,oY+dy-1]

    if(NOT Mosaic)then begin
        if(oX + dx GT geomInfo.scr_xsize*.9)then begin ; image off screen
            oX = 0
            oY =  oY + maxdy + 2
            maxdy = dy
        endif else oX = oX + dx + 2
    endif

endFOR

(*pStateInfo).reDrawn = !true
return, bounds
end




pro checkBackgnd, pStateInfo, nosearch = nosearch
; Routine to check for roi file for backgrounds
; will attempt to fetch backgrounds from files in the same directory automatically
; if that fails, it will allow the user to select a background manually
; TAF need to put in a way to set a default background file

if(not keyword_set(nosearch))then nosearch = !false

BkgPosList = TEMPORARY(WHERE((*(*pStateInfo).pFileScan).ImgType eq 2, BkgCount))

if(BkgCount EQ 0 and nosearch)then begin
    (*pStateInfo).BkgPos = -10
    return
endif

if(BkgCount EQ 0) then begin ; search for a background
    ShowMsg, (*pStateInfo).InstLabel, 'File Contains No Background Images, searching other files in this directory.'

    ; automatic search for backgrounds
    files = findFile((*pStateInfo).path + '*.roi', count=filescount)

    if(filescount EQ 0)then begin ; pathological case, always should have roi files in the folder, but go manual regardless
        needmanualbackground = !true
    endif else begin
        ; put in order
        files = files[sort(files)]
        print,' total files in folder=', filescount

        ; determine which is the current file
        thisFileindex = where(files eq (*pStateInfo).inFileName)
        print,'current file index=',files[thisFileindex[0]]

        if(filescount eq 1) then begin ; only one file in this directory, do manual search
            needmanualbackground = !true
        endif else begin
            needmanualbackground = !false
        endelse

        searchdir = -1
        searchfileindex = thisfileindex + searchdir ; start looking at the previous file, and searching backward
    endelse

    ; search files, starting backwards (earlier time) if available, then forward if available, then manually if none found
    foundBackground = !false

    while(not foundBackground) do begin
        print, 'in checkBackgnd loop, searching roi files for backgrounds'
        wait, .001
        if(needmanualbackground) then begin ; search for a background from a manually selected file
            ret  = dialog_message("Please select an .roi file for Manual Background Selection", /information)
            file = dialog_pickfile(title="Manual Background File Selection", filter='*.roi', path = (*pStateInfo).path)

            if(file ne '')then begin
                openr, lun, file, /get, /swap_if_big_endian

                ROI_INFO = ScanSPEC( lun, fileVers, labelID=(*pStateInfo).InstLabel) ;fileVers returns file version number or error code

                if(fileVers lt 0)then begin
                    r = dialog_message('Incorrect File Version or Currupt File, Please select another file.',/error)
                endif

                index = where(ROI_INFO.scanlist.ImgType gt 0, cnt)

                if(cnt gt 0)then begin ; there are images in the file
                    ImgscanList = ROI_INFO.scanlist[index]
                    backScanList = where(ImgscanList.imgtype eq 2, backCnt)

                    print, backcnt,' Backgrounds found in ', file

                    if(backcnt gt 0) then begin ; there are backgrounds in the file
                        position = backScanList[0]

                        if(cnt gt 1) then get_best_backGnd_pos, ImgscanList, backScanList, position

                        (*pStateInfo).pBackPosindx = ptr_new(backScanList,/no_copy)

                        ;get background frame
                        BackFrame = read_backGnd(lun,ROI_INFO.ROIHeader.Ver,ImgscanList[position].fpos, frameheader=frameheader)  ; contains the structure of the background

                        dx = BackFrame.Imageheader.ex - BackFrame.Imageheader.sx + 1
                        dy = BackFrame.Imageheader.ey - BackFrame.Imageheader.sy + 1

                        if((*pStateInfo).FileHead.DX EQ dx AND (*pStateInfo).FileHead.DY EQ dy) then begin

                            wset,(*pStateInfo).MViewID
                            tvscl,congrid(*(BackFrame.pBackImg),dx*.4,dy*.4)

                            r = dialog_message('Accept this as the background frame?',/question)

                            if(r eq 'Yes') then begin
                                foundBackground = !true
                                needmanualbackground = !false

                                (*pStateInfo).pBackPosindx = ptr_new(backScanList,/no_copy)
                                (*pStateInfo).BackFrame = ptr_new(BackFrame,/no_copy)
                                (*pStateInfo).BkgPos =  -2   ; when BkgPos = -2 then do not search for another background!
                                (*pStateInfo).Bkgs_exist = !true

                                r = dialog_message('Add this background to the end of the current file?',/question)

                                if(r eq 'Yes') then begin
                                    if(not (fstat((*pStateInfo).InFileLun)).write)then begin
                                        ; close and reopen as update
                                        free_lun,(*pStateInfo).InFileLun
                                        lun1 = 0
                                        openU,lun1,(*pStateInfo).InFileName,/get,error=err

                                        if(err ne 0) then begin
                                            msg = [['File not writeable.'],['Check to see if file is read only']]
                                            r = dialog_message(msg)
                                            print,'Background not saved to file!'
                                        endif else (*pStateInfo).InFileLun = lun1
                                    endif

                                    if(lun1 ne 0)then begin
                                        ; move pointer to the end of the file
                                        fpos = (fstat((*pStateInfo).InFileLun)).size
                                        point_lun,(*pStateInfo).InFileLun,fpos

                                        ; change time to equal last image time in file + 1 second
                                        frameheader.time = ImgscanList[n_elements(ImgScanList)-1].Htime + 1

                                        ; make sure imgtype is background
                                        frameheader.imgtype = frameheader.imgtype OR 2

                                        ; write a frameheader
                                        writeU,(*pStateInfo).InFileLun, frameheader
                                        writeU,(*pStateInfo).InFileLun,(*(*pStateInfo).BackFrame).imageheader
                                        writeU,(*pStateInfo).InFileLun,*(*(*pStateInfo).BackFrame).pBackImg
                                        print,'Background saved to file'
                                        BkgCount = 1
                                    endif
                                endif ; Yes to add background to end of file
                            endif else begin
                                ShowMsg, (*pStateInfo).InstLabel, 'Image must be of full size in order to be a background.'
                                r = dialog_message('Image must be of full size in order to be a background.')
                            endelse
                        endif
                    endif else print,'no backgrounds found, try another file'
                endif else print,'no images found, try another file'

                free_lun, lun
            endif ; file ne '' for manual background
        endif else begin ; manual selection not required (yet), attempting automatic file selection
            if(searchfileindex lt 0) then begin ; change search file direction
                searchdir = 1
                searchfileindex = thisfileindex + searchdir ; set to the next file after the one missing a background
            endif

            if(searchfileindex ge filescount-1) then begin ; at the last file in the directory, go manual
                needmanualbackground = !true
            endif

            if(not needmanualbackground) then begin ; check the current file index for a background
                print, 'searching file #', searchfileindex, ' named ', files[searchfileindex]

                openr, lun, files[searchfileindex], /get, /swap_if_big_endian

                ROI_INFO = ScanSPEC(lun, fileVers) ;fileVers returns file version number or error code

                if(fileVers ge 0)then begin ; fileVers lt 0
                    index = where(ROI_INFO.scanlist.ImgType eq 2, backcnt)

                    if(backcnt gt 0) then begin
                        ImgscanList = ROI_INFO.scanlist[index]
                        print, backcnt,' Backgrounds found in ', files[searchfileindex]

                        fpos = ROI_INFO.scanlist[index[0]].fpos

                        (*pStateInfo).pBackPosindx = ptr_new(backScanList,/no_copy)
                        BackFrame = read_backGnd(lun,ROI_INFO.ROIHeader.Ver,fpos, frameheader=frameheader)  ; contains the structure of the background

                        testimage = *backframe.pbackimg

                        if(n_elements(testimage) le 1) then begin
                            print, 'Error in gui_lib.pro checkBackgnd, *backframe.pbackimg is incorrect, skipping bad background, continuing search'
                        endif else begin
                            (*pStateInfo).BackFrame = ptr_new(BackFrame,/no_copy)
                            (*pStateInfo).BkgPos =   -2    ; when BkgPos = -2 then do not search for another background!
                            (*pStateInfo).Bkgs_exist = !true

                            foundBackground = !true
                        endelse
                    endif
                endif

                free_lun,lun

                searchfileindex = searchfileindex + searchdir
            endif
        endelse ; manual vs automatic
    endwhile ; foundBackground true to exit while
endif else (*pStateInfo).BkgPos = BkgPosList[0]

msg = STRING((*pStateInfo).TotImgCount, ' Images, ', BkgCount, ' Bkgs, ','    DX:',(*pStateInfo).FileHead.DX, ' DY:', (*pStateInfo).FileHead.DY)
ShowMsg, (*pStateInfo).InstLabel,msg
end ; checkBackgnd



;*******************************************************************************
;*****  add this special for namma archiving which just use a fixed background  if a roi don'y have a background **************
;***************************************************************************************************
pro checkBackgnd_namma, pStateInfo, nosearch = nosearch
; Routine to check for backgrounds
if(not keyword_set(nosearch))then nosearch = !false

BkgPosList = TEMPORARY(WHERE((*(*pStateInfo).pFileScan).ImgType eq 2, BkgCount))
if(BkgCount EQ 0 and nosearch)then begin
    (*pStateInfo).BkgPos = -10
    return
endif

if( BkgCount EQ 0 )then begin
    files ='C:\08150326.roi' ; give a specififi file for the background.
    openr,lun,files,/get,/swap_if_big_endian

    ROI_INFO = ScanSPEC( lun, fileVers) ;fileVers returns file version number or error code
    index = where(ROI_INFO.scanlist.ImgType eq 2,backcnt)

    if(backcnt gt 0)then begin

        ImgscanList = ROI_INFO.scanlist[Index]
        print,backcnt,' Backgrounds found'
        noBackground = !false
    endif

    if(backcnt gt 1 )then fpos = ROI_INFO.scanlist[index[backcnt-1]].fpos else fpos = ROI_INFO.scanlist[index[0]].fpos

    (*pStateInfo).pBackPosindx = ptr_new(backScanList,/no_copy)
    BackFrame = read_backGnd(lun,ROI_INFO.ROIHeader.Ver,fpos, frameheader=frameheader)  ; contains the structure of the background

    dx = BackFrame.Imageheader.ex - BackFrame.Imageheader.sx + 1
    dy = BackFrame.Imageheader.ey - BackFrame.Imageheader.sy + 1

    (*pStateInfo).BackFrame = ptr_new(BackFrame,/no_copy)
    (*pStateInfo).BkgPos =   -2    ; when BkgPos = -2 then do not search for another background!
    (*pStateInfo).Bkgs_exist = !true
    free_lun,lun
    if(not (fstat((*pStateInfo).InFileLun)).write)then begin
        ; close and reopen as update
        free_lun,(*pStateInfo).InFileLun
        lun1 = 0
        openU,lun1,(*pStateInfo).InFileName,/get,error=err
        if(err ne 0)then begin
            msg = [['File not writeable.'],['Check to see if file is read only']]
            r = dialog_message(msg)
            print,'Background not saved to file!'
        endif else (*pStateInfo).InFileLun = lun1

    endif

    if(lun1 ne 0)then begin
        ; move pointer to the end of the file
        fpos = (fstat((*pStateInfo).InFileLun)).size
        point_lun,(*pStateInfo).InFileLun,fpos

        ; change time to equal last image time in file + 1 second
        frameheader.time = ImgscanList[n_elements(ImgScanList)-1].Htime + 1

        ; make sure imgtype is background
        frameheader.imgtype = frameheader.imgtype OR 2

        ; write a frameheader
        writeU,(*pStateInfo).InFileLun, frameheader
        writeU,(*pStateInfo).InFileLun,(*(*pStateInfo).BackFrame).imageheader
        writeU,(*pStateInfo).InFileLun,*(*(*pStateInfo).BackFrame).pBackImg
        print,'Background saved to file'
    endif
endif else (*pStateInfo).BkgPos = BkgPosList[0]; end of if bkgcount eq 0
end; end for the check_background_namma.
;**********************************************************************************************

;**********************************************************************************************
pro LoadFile, pStateInfo, filename = filename

; Purpose ;to load a new file, selected by the user, and setup many parameters and the GUI
; Parameters
;   pStateInfo  Main state structure
;keyword
;   filename  if defined, this is file name to be opened

;   forward_function checkBackgnd

if((*pStateInfo).demo)then begin
    ; limit access in demo mode
    FileName = 'Demo.roi'
    err = 0
    OpenU, DFileUnit, FileName, /Get_lun, ERROR = err, /BUFSIZE,/swap_if_big_endian

endif else begin

    if(not keyword_set(filename))then begin
        ; get filename from user interaction
        if((*pStateInfo).path eq '')then begin
            FileName = DIALOG_PICKFILE(FILTER='*.roi', TITLE="Select a SPEC Data File", get_path = path, /MUST_EXIST);,/multiple)
        endif else begin
            FileName = DIALOG_PICKFILE(FILTER='*.roi', TITLE="Select a SPEC Data File", path = (*pStateInfo).path, /MUST_EXIST,get_path = path);,/multiple)
        endelse

        ; bail out if user clicked cancel
        if (FileName[0] eq '') then return

        if((*pStateInfo).path eq '' or (*pStateInfo).path ne path)then (*pStateInfo).path = path

        ; remove old file info
        clean_up,pStateInfo
    endif else    clean_up,pStateInfo,/processing

    CNT = n_elements(filename)
    if(cnt gt 1)then r = dialog_message('Multi File not implemented yet',/info)

    err = 0
    OpenR, DFileUnit, FileName, /Get_lun, ERROR = err, /BUFSIZE,/swap_if_big_endian

endelse


if (err NE 0) then begin
    PRINT, !ERR_STRING
    DFileUnit = -1
    ShowMsg, (*pStateInfo).InstLabel,'Error from opening file'
    erase
    xyouts,.02,.40,'ERROR... see log window',/normal,charsize=6.0,font=0,color=255
    return
endif

(*pStateInfo).InFileLun = DFileUnit
(*pStateInfo).InFileName = FileName;[i]

WIDGET_CONTROL, /HOURGLASS
wset,(*pStateInfo).MViewID
ERASE
xyouts,.02,.40,'Scanning File for frame locations... please wait.',/normal,charsize=6.0,font=0,color=255


;fileVers returns file version number or error code
HKPath = STRMID((*pStateInfo).InFileName, 0, RSTRPOS((*pStateInfo).InFileName, '.'))
ROI_INFO = ScanSPEC( DFileUnit, fileVers, scaleFactor = (*pStateInfo).scaleFactor, /reorder, labelID = (*pStateInfo).InstLabel,HKLOG_request=(*pStateInfo).HKLOG_request,path = HKPath)

if(fileVers lt 0)then begin
    ; fatal error
    r = dialog_message(' Fatal Error',/error)
    return
endif


if(size(roi_Info,/type) ne 8)then return
; If Successful this returns a structure with 4 elements : {ROIHeader,scanList,data}
;   ROIHeader: is the overall file header of the form
;       {SPECHead, Ver:0, Year:0, Month:0, DX:0, DY:0, Text:BYTARR(70)}
;   scanlist:   an array of structures with the following elements, size determined by file
;       FPos:   file pointer for start of frame header
;       BlockMark  type of block read
;       ImgType:   bitwise variable with image type info
;       HTime:     time of most recent housekeeping block
;       day:     time elements of block
;       hour:   "   "
;       minute:       " "
;       sec:     "    "
;       msec:   "   "
;   data: contains a fltarr of the concentration


(*pStateInfo).pConc = ROI_Info.data.conc
IF PTR_VALID((*pStateInfo).pConc) THEN BEGIN
    wset,(*pStateInfo).plotID
    plot,*((*pStateInfo).pConc),xstyle = 1,ystyle=4,psym=10,ymargin=[2,0],xmargin=[0.3,0.3],color=254
ENDIF

; roi_info must be a structure
if(size(roi_info,/type) ne 8)then return

(*pStateInfo).FileHead = ROI_INFO.ROIHeader

; determine cross platform directory seperator
case strlowcase(!version.os_family) of
    'macos':seperator = ':'
    'unix':seperator = '/'
    'vms':
    'windows':seperator='\'
endcase
(*pStateInfo).seperator = seperator

; TAF 2014-03-28 this section needs error handling so that it can handled filenames that people have manually changed

; get file date from file name
nameStart = rstrpos(filename,seperator)+1
(*pStateInfo).date.year = (*pStateInfo).FileHead.year
(*pStateInfo).date.month =(*pStateInfo).FileHead.month;  ; mo changed this to be always correct, becaus efilename can be 20120511*.roi or 0511*.roi. ; fix(strmid(filename,nameStart,2))
basefile=file_basename(filename)
test_year=fix(strmid(basefile,0,4))

if test_year eq  (*pStateInfo).date.year then begin;   ; mo added this for two type of file names.
    (*pStateInfo).date.day=fix(strmid(basefile,6,2))
endif else begin
    (*pStateInfo).date.day = fix(strmid(filename,nameStart+2,2))
endelse


if(size(ROI_INFO.scanlist,/type) eq 2)then begin   ; error condition
    ; try to continue
    ;message,'Error from scanSPEC',/ioerror
    ShowMsg, (*pStateInfo).InstLabel,'Error from scanSPEC'
    wset,(*pStateInfo).plotID
    erase
    xyouts,.02,.40,'ERROR... see log window',/normal,charsize=6.0,font=0,color=255
    return
endif



if(ptr_valid((*pStateInfo).pFileScan))then PTR_FREE, (*pStateInfo).pFileScan
(*pStateInfo).pFileScan = ptr_new(TEMPORARY(ROI_INFO.ScanList))

; this block of code searches for frames with times that are out of sequence.
; It then sets the time to the previous and marks the frame as bad.
imgListindex = where((*(*pStateInfo).pFileScan).imgtype ge 0)
imgList = (*(*pStateInfo).pFileScan)[imgListindex]
allTimes = imglist.HTime ; double(imglist.HTime) + double(imglist.msec)/1000.0D

for k=0L,n_elements(allTimes)-2 do begin
    if(alltimes[k+1] lt alltimes[k])then begin
        (*(*pStateInfo).pFileScan)[imgListindex[k]].HTime = alltimes[k+1]
        (*(*pStateInfo).pFileScan)[imgListindex[k]].badframe = 1
        ShowMsg, (*pStateInfo).InstLabel,'Frame# '+strtrim(k,2)+' marked as bad... time out of sequence.'
        wset,(*pStateInfo).plotID
        erase
        xyouts,.02,.40,'WARNING... see log window',/normal,charsize=6.0,font=0,color=255
    endif
endfor


frameData = read_frame(DFileUnit,ROI_INFO.ROIHeader.Ver,((*(*pStateInfo).pFileScan)[(*pStateInfo).FirstImgPos].fpos),date=(*pStateInfo).date, TSBlanked = ((*(*pStateInfo).pFileScan)[(*pStateInfo).FirstImgPos].TSBlanked))

;added this to first get the roi_frame_header_version number first and store it away before reading obj-file
(*pstateINFo).roi_frameheader_version=frameData.frameheader.ver  ; this could be 40, 30, or something.
if frameData.frameheader.ver eq 40 then (*pstateINFo).scaleFactor=0.000341333 else (*pstateINFo).scaleFactor=0.000682667
;above is added by mo  on january 8 2004

; try to get an object file
if((*pStateInfo).AUTOEXTRACT)then Load_Object_file,pStateInfo

; do for 1st file only
;if(i eq 0)then begin

; set gui to special setting values and set FirstImgPos@compile_extrac
set_gui,pStateInfo

; first frame located at (*(*pStateInfo).pImgPosindx)[0] which is stored in (*pStateInfo).FirstImgPos
; get the frame indicated via position in scan list


frameData = read_frame(DFileUnit,ROI_INFO.ROIHeader.Ver,((*(*pStateInfo).pFileScan)[(*pStateInfo).FirstImgPos].fpos),date=(*pStateInfo).date, TSBlanked = ((*(*pStateInfo).pFileScan)[(*pStateInfo).FirstImgPos].TSBlanked))
; Return value
;     structure of form {FrameHeader:FrameHeader,pFrame:pFrame}
;     where
;        FrameHeader is a structure containing the frame header information. See definitions in read_frame.pro
;        pFrame is a pointer to a array of structures containing the particle information.  The form is
;        {ImageHeader:ImageHeader, pROI:PTR_NEW(/allocate)}
;        where
;          ImageHeader is described in read_frame.pro
;          pROI points to an array containing the actual particle data


if(size(frameData,/type) eq 8)then begin

    ;place time into GUI
    frame_time = convertFromSeconds(frameData.frameheader.Time,(*pStateInfo).fileHead.year)
    frame_time.msec  = ROUND(CalcStrobeTime(frameData.frameheader.ArrivalT1, frameData.frameheader.ArrivalT2,frameData.frameheader.Ver))

    ; mo added following to fixed  3VCPI times.
    ;***************************************************************
    ; above Frame_time is for CPI  time.  But FrameTime calculated in Process_lib2.pro is the DAS time.

    ;******************************************************************



    time1= 'CPI: ' + strtrim(fix(frame_time.hour),2) + ':' + strtrim(fix(frame_time.minute),2) + ':'+  strtrim(fix(frame_time.sec),2) + ':'+  strtrim(fix(frame_time.msec),2)
    time2= 'DAS: ' + strtrim(fix(frameData.frameheader.hour),2) + ':' + strtrim(fix(frameData.frameheader.minute),2) + ':'+  strtrim(fix(frameData.frameheader.sec),2) + ':'+  strtrim(fix(frameData.frameheader.msec),2)
    widget_control,(*pStateInfo).widgets.time1,set_value = time1
    widget_control,(*pStateInfo).widgets.time2,set_value = time2

    ; save into state structure pFrameData always holds the current frame
    (*pStateInfo).pFrameData = ptr_new(frameData,/no_copy)

    ; if available, get obj data for this frame
    if(ptr_valid((*pStateInfo).pObjList))then begin
        objData = get_obj_header((*pStateInfo).ObjLUN,(*(*pStateInfo).pObjList)[(*pStateInfo).ImgPos-1].fPos)
        ; store obj info
        (*pStateInfo).pObjData = ptr_new(objData)
    endif


    ; check to see if a background image is available
    ; checkBackgnd_namma,pStateInfo
    checkBackgnd,pStateInfo;        : this is back to normal,  on April 3, 2008.


    (*pStateInfo).backRemoved = !false

    if((*pStateInfo).BkgPos ge 0)then begin  ; BkgPos negative means either no background or we are using an external backgrnd
        ; GET A BACKGROUND

        ImgScanList = (*(*pStateInfo).pFileScan)[*(*pStateInfo).pImgPosindx]
        BackPosindex = where(ImgScanList.imgtype eq 2,cnt)
        position = 1       ;look near beginning of file

        if(not ptr_valid((*pStateInfo).pBackPosindx))then begin

            if(cnt gt 0)then begin
                print,cnt,' Backgrounds found'
                position = BackPosindex[0]
                if(cnt gt 1)then get_best_backGnd_pos,ImgScanList,BackPosindex,position

                (*pStateInfo).pBackPosindx = ptr_new(BackPosindex,/no_copy)
                (*pStateInfo).Bkgs_exist = !true
            endif else print,'no backgrounds found'

        endif else get_best_backGnd_pos, imgScan, BackPosindex, position

        ShowMsg, (*pStateInfo).InstLabel,'Current Background at Frame # '+ strtrim(position+1,2)

        BackFrame = read_backGnd((*pStateInfo).InFileLun,ROI_INFO.ROIHeader.Ver,ImgScanList[position].fpos)     ; contains the structure of the background

        if(size(BackFrame,/type) eq 8)then begin
            (*pStateInfo).BackFrame = ptr_new(BackFrame,/no_copy)
            (*pStateInfo).BkgPos = position                 ; contains the zero-based index number into scanList of the background to use
        endif
    endif

    ; set min/max to gui
    set_min_max, pStateInfo

    ; Show main info line
    disp_frame_info, pStateInfo, 0

    ; now display frame
    bounds = DispFrame(pStateInfo)
    (*pStateInfo).pBounds =  ptr_new(bounds)

    ; update title bar om tlb with file name
    ;title = (*pStateInfo).title + '  current file: ' + (*pStateInfo).infilename
    ;widget_control,(*pStateInfo).widgets.tlb, tlb_set_title = title
    widget_control,(*pStateInfo).fileLabel,set_value = 'Current file: ' + (*pStateInfo).infilename
endif else begin
    print,'Invalid Frame'
endelse

;endif
;endfor
end


;===============================================================================
pro set_TLB_title,pStateInfo

    FNameLen = STRLEN((*pStateInfo).InFileName)

    ; Extract path
    NamePos = RSTRPOS((*pStateInfo).InFileName, '\')
    ;(*pStateInfo).FileListPath = STRMID((*pStateInfo).InFileName, 0, NamePos +1)

    ; Here we prepare to display the file name on the screen.
    Pieces = STR_SEP((*pStateInfo).InFileName, '\')
    PieceCount = N_ELEMENTS(Pieces)
    RemCount = 1
    FName = (*pStateInfo).InFileName
    WHILE FNameLen GT 70 DO begin
        if PieceCount - RemCount GT 1 then begin
            FName = ''
            FOR k=0, PieceCount -2 -RemCount DO FName = FName + Pieces(k)+'\'
            FName = FName + '...\' + Pieces(PieceCount -1)
        endif else FName = '...' + STRMID((*pStateInfo).InFileName, FNameLen -67, 66)
        RemCount = RemCount +1
        FNameLen = STRLEN(FName)
    endWHILE
    ;WIDGET_CONTROL, (*pStateInfo).FileNameLab, TLB_SET_TITLE=(*pStateInfo).Title +":  " + FName

end



;===============================================================================
Pro get_best_backGnd_pos, scanList, index, position
    ;   purpose
    ;     search for the best background, given a frame index

    ;   scanList: Input      Image file scan list
    ;   index:       Input        indices (into Image scanList) of background images
    ;   Position:   Input/Output    position in ScanList to start search for background
    ;                    Note position is zero based!

    ; return
    ;   best position in input parameter

    if(position eq -2)then return  ; this means that the background was retrieved from another file
    cnt = n_elements(index)

    ;         bkg1    bkg2     Position   bkg3   bkg4
    ;             fPos_below  fPos     fPos_above
    ; -------------X-------X----------I-----------X--------X-----------   file position plot

    ; Consider three cases, 1: position before all backgrounds, 2: position after all backgrounds, 3: position within backgrounds
    case 1 of
        (position le index[0]): position = index[0]
        (position ge index[cnt-1]): position = index[cnt-1]
        else:begin

        idx = where(position ge index,count)

        ; locate some points in the file
        fPos_above = scanList[index[count]].fPos
        fPos_below = scanList[index[count-1]].fPos
        fPos = scanList[Position].fPos

        ; determine which is closest
        delta_low = abs(fPos-fPos_below)
        delta_high = abs(fPos_above-fPos)

        position = (delta_low le delta_high) ? index[count-1] : index[count]

    end
endcase
end

;===============================================================================
pro set_min_max, pStateInfo

    if((*pStateInfo).AutoScale)then begin
        ; ROIArray is an array of pointers for each particle in the frame
        ROIArray = *(*(*pStateInfo).pFrameData).pFrame

        ; How many ROIs?
        n_imgs = n_elements(  ROIArray )
        frameMin = 255
        frameMax = 0
        backRemoved = (*pStateInfo).backRemoved

        ; for debugging grayscale in 20140821233145.roi
        ;if(!stop_from_here_on eq 1) then stop

        for i = 0,n_imgs - 1 do begin

            if((*pStateInfo).SubBkgd and not (*pStateInfo).backRemoved)then begin
                ROImax = 255
                ROImin = 0

                ; subtract the current background image from the particles
                if(ptr_valid((*pStateInfo).BackFrame))then begin

                    pBackImg = (*(*pStateInfo).BackFrame).pBackImg
                    backROI = (*pBackImg)[ROIArray[i].ImageHeader.sx:ROIArray[i].ImageHeader.ex,ROIArray[i].ImageHeader.sy:ROIArray[i].ImageHeader.ey]
                    ; save this back into the data array
                    temp = *ROIArray[i].pROI - fix(backROI)
                    ROImin = min(temp, max = ROImax)
                    if(ROImin lt 0)then begin
                        *ROIArray[i].pROI = bytscl(temp - ROImin, top = !d.table_size - 3) + 1B
                        ROImax = ROImax - ROImin
                        ROImin = 0
                    endif else *ROIArray[i].pROI = bytscl(temp, top = !d.table_size - 3) + 1B
                    backRemoved = !true
                endif else backRemoved = !false
            endif else *ROIArray[i].pROI = bytscl(*ROIArray[i].pROI,top = !d.table_size - 3)+ 1B ;ROImin = min(*ROIArray[i].pROI, max = ROImax)

            ;frameMin = ROIMin < frameMin
            ;frameMax = ROIMax > frameMax
        endfor

        (*pStateInfo).backRemoved = backRemoved

        ;frameMin = frameMin > 0
        ;frameMax = frameMax < 255
        frameMin = 1
        frameMax = !d.table_size - 2

        WIDGET_CONTROL, (*pStateInfo).ScaleMinID, SET_SLIDER_MIN=frameMin
        WIDGET_CONTROL, (*pStateInfo).ScaleMinID, SET_SLIDER_MAX=frameMax - 2
        WIDGET_CONTROL, (*pStateInfo).ScaleMinID, SET_VALUE=frameMin

        WIDGET_CONTROL, (*pStateInfo).ScaleMaxID, SET_SLIDER_MIN=frameMin + 2
        WIDGET_CONTROL, (*pStateInfo).ScaleMaxID, SET_SLIDER_MAX=frameMax
        WIDGET_CONTROL, (*pStateInfo).ScaleMaxID, SET_VALUE=frameMax

    endif else begin
        WIDGET_CONTROL, (*pStateInfo).ScaleMinID, SET_SLIDER_MIN=0
        WIDGET_CONTROL, (*pStateInfo).ScaleMinID, SET_SLIDER_MAX=253
        ;WIDGET_CONTROL, (*pStateInfo).ScaleMinID, SET_VALUE=0

        WIDGET_CONTROL, (*pStateInfo).ScaleMaxID, SET_SLIDER_MIN=2
        WIDGET_CONTROL, (*pStateInfo).ScaleMaxID, SET_SLIDER_MAX=255
        ;WIDGET_CONTROL, (*pStateInfo).ScaleMaxID, SET_VALUE=255

    endelse

end


pro delete_frame,FrameData
    ;Purpose - to delete the currant heap variable associated with a frame

    ;   Parameter
    ;     FrameData    Input  regular variable or pointer to current frame

    if(size(FrameData,/type) eq 10)then begin
        if ptr_valid(FrameData)then begin
            ; check for obj version and delete pointers
            for i=0,n_elements(*(*FrameData).pFrame)-1 do    ptr_free,(*(*FrameData).pFrame)[i].pROI
            if((*FrameData).frameHeader.Ver eq 30)then begin
                ptr_free,(*(*FrameData).pFrame).imageheader.Harmonics
                ptr_free,(*(*FrameData).pFrame).imageheader.Boundary
            endif
            ptr_free,(*FrameData).pFrame
            ptr_free,FrameData
        endif
    endif else begin
        ; check for obj version and delete pointers

        if ptr_valid(Framedata.pFrame) gt 0 then begin
            for i=0,n_elements(*FrameData.pFrame)-1 do    ptr_free,(*FrameData.pFrame)[i].pROI
        endif

        if(FrameData.frameHeader.Ver eq 30)then begin
            ptr_free,(*FrameData.pFrame).imageheader.Harmonics
            ptr_free,(*FrameData.pFrame).imageheader.Boundary
        endif

        ptr_free,FrameData.pFrame

    endelse

end


pro clean_up, pStateInfo, processing=processing

    ; purpose : to clean up memory when a new file is loaded or at program end
    ; keyword
    ; processing  if set, then graph processing is happening and the ptime Info should no be deleted

    if ptr_valid((*pStateInfo).pObjList) then PTR_FREE, (*pStateInfo).pObjlist
    if(ptr_valid((*pStateInfo).pObjData)) then begin
        ptr_free,(*(*pStateInfo).pObjData).pFrame
        ptr_free,(*(*pStateInfo).pObjData).pImHDfPos
        ptr_free,(*pStateInfo).pObjData
    endif
    if((*pStateInfo).ObjLUN ne -1)then begin
        free_lun,(*pStateInfo).ObjLUN
        (*pStateInfo).ObjFile = ''
        (*pStateInfo).ObjLUN = -1
    endif

    if (*pStateInfo).InFileLun GT 0 then FREE_LUN, (*pStateInfo).InFileLun

    if(not keyword_set(processing))then processing = !false
    ; cleanup the linked list from generate/view plots
    if(ptr_valid((*pStateInfo).pTimeInfo) and not processing)then begin
        if(ptr_valid((*(*pStateInfo).pTimeInfo).pCurrent_link))then begin
            pLink = find_last_link( (*(*pStateInfo).pTimeInfo).pCurrent_link )
            while ( ptr_valid(pLink)) do pLink = delete_link(pLink)
        endif
    endif
    if ptr_valid((*pStateInfo).pFileScan) then PTR_FREE, (*pStateInfo).pFileScan
    if(ptr_valid((*pStateInfo).pBounds))then ptr_free,(*pStateInfo).pBounds
    if ptr_valid((*pStateInfo).pFrameData)then delete_frame,(*pStateInfo).pFrameData
    if(ptr_valid((*pStateInfo).pConc))then ptr_free,(*pStateInfo).pConc
    if(ptr_valid((*pStateInfo).pImgPosindx))then ptr_free,(*pStateInfo).pImgPosindx

    if(not processing)then begin
        if obj_valid((*pStateInfo).PrintROI.View) then OBJ_DESTROY, (*pStateInfo).PrintROI.View
        if obj_valid((*pStateInfo).PrintROI.Font1) then OBJ_DESTROY, (*pStateInfo).PrintROI.Font1
        if obj_valid((*pStateInfo).PrintROI.TimeText) then OBJ_DESTROY, (*pStateInfo).PrintROI.TimeText
        if obj_valid((*pStateInfo).PrintROI.ValueText) then OBJ_DESTROY, (*pStateInfo).PrintROI.ValueText
    endif

    if ptr_valid((*pStateInfo).BackFrame) then begin
        ptr_free,(*(*pStateInfo).BackFrame).pBackImg
        ptr_free,(*pStateInfo).BackFrame
    endif

    if(ptr_valid((*pStateInfo).pTimeInfo) and not processing)then begin
        ptr_free,(*pStateInfo).pTimeInfo
        (*pStateInfo).pTimeInfo = ptr_new()
    endif
    if ptr_valid((*pStateInfo).pBackPosindx) then ptr_free, (*pStateInfo).pBackPosindx
    (*pStateInfo).pBackPosindx = ptr_new()

end


pro UpdateBlock, pStateInfo
    ; purpose : to update one block of data

    ; parameters
    ;   pStateInfo

    ; frame is a structure describing the frame
    frame = (*(*pStateInfo).pFrameData)

    ;print,'Updating frame #',(*pStateInfo).imgpos,' at fpos ',ImgList[(*pStateInfo).imgpos-1].fpos
    ; close and reopen file for output
    free_lun,(*pStateInfo).InFileLUN
    err = 0
    openu,lun,(*pStateInfo).InFileName,/get,error = err
    if(err ne 0)then begin
        r = dialog_message('Error opening file for write... verify file is writable.',/info)
        return
    endif
    (*pStateInfo).InFileLUN = lun

    ;locate file pointer for writing
    ImgList = (*(*pStateInfo).pFileScan)[*(*pStateInfo).pImgPosindx]
    point_lun,(*pStateInfo).InFileLUN,ImgList[(*pStateInfo).imgpos-1].fpos

    ;write frame header
    writeu,(*pStateInfo).InFileLUN,frame.frameheader

    ; loop through rois
    for j = 0,frame.FrameHeader.ROIsCount - 1 do begin
        writeu,(*pStateInfo).InFileLUN,(*frame.pFrame)[j].ImageHeader
        ;print,'write Particle# ',j,' flags= ',(*frame.pFrame)[j].ImageHeader.flags,format='(a16,x,i3,a8,x,z4)'
        ;print,'write Dim of image = ',size(*(*frame.pFrame)[j].pROI)

        ;if((*pStateInfo).fullFile) then writeu,(*pStateInfo).InFileLUN,*((*frame.pFrame)[j].pROI) else begin

        ; advance pointer over image data
        dim = size(*((*frame.pFrame)[j].pROI))
        case dim[3] of
            1:bytesPerPix =1
            2:bytesPerPix =2
            3:bytesPerPix =4
            else:
        endcase
        fpos = (fstat((*pStateInfo).InFileLUN)).cur_Ptr + dim[4]*bytesPerPix
        point_lun,(*pStateInfo).InFileLUN,fpos

        ;endelse
    endfor

    ; reset update flag
    (*pStateInfo).updatefile = !false

    ; close and reopen file for read only
    free_lun,(*pStateInfo).InFileLUN
    openR,lun,(*pStateInfo).InFileName,/get
    (*pStateInfo).InFileLUN = lun

end

pro reject_all, pStateInfo

    ; frame is a structure describing the frame
    frame = (*(*pStateInfo).pFrameData)

    ; loop through rois and mark all as rejected
    for j = 0,frame.FrameHeader.ROIsCount - 1 do begin
        (*frame.pFrame)[j].ImageHeader.flags = (*frame.pFrame)[j].ImageHeader.flags or '8000'x
        if(ptr_valid((*pStateInfo).pObjData))then begin
            (*(*(*pStateInfo).pObjData).pFrame)[j].imageheader.P1 = 1.0
            update_obj_header1,(*pStateInfo).ObjLUN, (*(*(*pStateInfo).pObjData).pImHDfPos)[j], (*(*(*pStateInfo).pObjData).pFrame)[j].imageheader
        endif
    endfor

end

;===============================================================================
; ShowMsg: Utility to display scrollable messages.
PRO ShowMsg, Msg_wid, Msg

    if msg_wid eq 0 then return;  added this on spetember 1, 2009.

    WIDGET_CONTROL, Msg_wid, GET_VALUE=MsgsNow
    Lines = N_ELEMENTS(MsgsNow)
    if Lines GT 40 then begin
        MsgsNow = MsgsNow(Lines -38:Lines-1)
        Lines = 38
    endif
    WIDGET_CONTROL, Msg_wid, SET_VALUE=MsgsNow
    WIDGET_CONTROL, Msg_wid, /APPend, SET_VALUE=Msg
    WIDGET_CONTROL, Msg_wid, set_text_top_line= Lines -2 > 0

end

;===============================================================================
Pro disp_frame_info, pStateInfo, index, full = full,  data = data

; Purpose to display information on the GUI about the particle under the mouse
; Parameters
;   pStateInfo:   state structure
;   frame:       indicates which frame we are in
;   index:       indicates which ROI of the frame
;   data     structure used for getting the value

; Keyword:
;   full: controls if all information is displayed, should be set if over a particle
;   pImg  output  returns pointer to img array
; returns the actual image array

;read the frame number from the GUI
widget_control,(*pStateInfo).ImgSliderID,get_value = frame

wset,(*pStateInfo).MViewID

if((*pStateInfo).DispMode gt 0)then begin
    ; get settings from gui
    gradient = ((*pStateInfo).DispMode and 2)/2;  if gradient is checked, the gradient =1.

    img = Disp_filt(*((*(*(*pStateInfo).pFrameData).pFrame)[index] ).pROI, gradient=gradient)
endif else img = *((*(*(*pStateInfo).pFrameData).pFrame)[index] ).pROI

if(keyword_Set(data))then data.val = (rotate(img,7))[data.dx,data.dy]

if(keyword_set(full))then begin
    imageHeader = (*(*(*pStateInfo).pFrameData).pFrame)[index].imageheader

    types = (*pStateInfo).types

    ; form information
    focus = strtrim(100-(imageHeader.Flags AND '00FF'X),2)
    crystal = ISHFT(imageHeader.Flags, -8) AND '001F'X
    cutoff = strtrim(fix(100 * (ISHFT(imageHeader.Dark,-8) AND '1F'X)/31.0),2)
    reject = ishft(imageheader.flags,-15)? 'YES' : 'NO'
    area = ''
    perimeter = ''
    roundness = ''
    length = ''
    width = ''
    roisize = ''
    pixsize = (*pStateInfo).Settings.PixSize

    ; if an Object file exists, then use it
    imgtype = (*(*pStateInfo).pFileScan)[(*(*pStateInfo).pImgPosindx)[(*pStateInfo).ImgPos-1]].imgtype
    if(ptr_valid((*pStateInfo).pObjList) and (imgtype eq 1))then begin

        ;pixsize = (*pStateInfo).Settings.PixSize
        roisize = string(format='(f6.1,a4,f6.1)',(imageHeader.len * pixsize),' x ',(imageHeader.wid *pixsize))

        ; read a frame of info from the obj file
        ;objData = get_obj_header((*pStateInfo).ObjLUN,(*(*pStateInfo).pObjList)[frame-1].fPos)
        objData = (*(*pStateInfo).pObjData)
        polyfill,[.02,.25,.25,.02,.02],[.40,.40,.5,.5,.40],/normal, color=0
        if(size(objData,/type) eq 8 )then begin

            tvlct,255,255,0,255
            focus = strtrim(fix((*objData.pFrame)[index].imageheader.focus) ,2)
            cutoff = strtrim(fix((*objData.pFrame)[index].imageheader.cutoff),2)
            habit = ((*objData.pFrame)[index].imageheader.crystal)
            ;print, "habit = " , habit
            area      = strtrim(long((*objData.pFrame)[index].imageheader.area  * pixsize * pixsize), 2);+' microns'
            perimeter = strtrim( fix((*objData.pFrame)[index].imageheader.perim * pixsize), 2);+' microns'

            ; roundness = string((*objData.pFrame)[index].imageheader.roundness, format='(f4.2)')
            ; TAF v 1.0.58, updating roundness text field to display 3 roundness values
            roundness = string(format='(%"%4.2f, %4.2f, %5.3f")', (*objData.pFrame)[index].imageheader.Roundness1, (*objData.pFrame)[index].imageheader.Roundness2, (*objData.pFrame)[index].imageheader.Roundness3)

            roisize = string(format='(f6.1,a4,f6.1)',((*objData.pFrame)[index].imageHeader.len * pixsize),' x ',((*objData.pFrame)[index].imageHeader.wid *pixsize))

            H2dsID = ((*(*PstateInfo).PFRAMEDATA).frameHeader).Pheight1;     ; === H2DSID
            V2dsID = ((*(*PstateInfo).PFRAMEDATA).frameHeader).Pheight2;     ; === V2DSID
            h2dsID = string(format='(%"0x%04x")', H2dsID)
            v2dsID = string(format='(%"0x%04x")', V2dsID)

            ; TAF disabling this change from reject to sphere for 3V-CPI
            ;if(habit eq 0 and area gt 0)then habit = 1

            ;reject = 'NO'
            ;print,habit
            ;this change handles old obj files that had water type = 0
            if(habit eq 1)then crystal = 'water' else crystal = types[habit]

            if((*objData.pFrame)[index].imageheader.P1 eq 1)then crystal1 = crystal + '  (manually set)' else crystal1 = crystal

            ;if(not (*pStateInfo).Settings.Mosaic)then begin
            polyfill,[.02,.25,.25,.02,.02],[.40,.40,.5,.5,.40],/normal, color=0
            if((*objData.pFrame)[index].imageheader.area ne 0)then begin
                length= strtrim(fix((*objData.pFrame)[index].imageheader.len*pixsize),2)
                width = strtrim(fix((*objData.pFrame)[index].imageheader.wid*pixsize),2)
                ;xyouts,.02,.44,'Length:' + strtrim(fix((*objData.pFrame)[index].imageheader.len*pixsize),2)+' microns',/normal,charsize=2.0,font=0,color=255
                ;xyouts,.02,.42,'Width:' + strtrim(fix((*objData.pFrame)[index].imageheader.wid*pixsize),2)+' microns',/normal,charsize=2.0,font=0,color=255
                crystal = (*pStateInfo).crystalName[habit]

                ;print,reject,(*objData.pFrame)[index].imageheader.P1
                if(reject eq 'YES' and (*objData.pFrame)[index].imageheader.P1)then begin
                    xyouts,.02,.42,'Particle Manually Rejected',/normal,charsize=2.0,charthick=1.3,font=0,color=255
                    ;crystal = 'Reject'
                    crystal = (*pStateInfo).crystalName[habit]
                endif else reject = 'NO'

                ;xyouts,.02,.40,'Particle Type: ' + crystal1,/normal,charsize=2.0,font=0,color=255
            endif else begin
                if((*objData.pFrame)[index].imageheader.P1 eq 1)then begin
                    xyouts,.02,.42,'Manual Setting',/normal,charsize=2.0,charthick=1.3,font=0,color=255
                    xyouts,.02,.44,'Particle Type: ' + crystal1,/normal,charsize=2.0,font=0,color=255
                endif else begin
                    xyouts,.02,.42,'Particle Rejected',/normal,charsize=2.0,charthick=1.3,font=0,color=255
                    crystal = 'Reject'
                    reject = 'YES'
                endelse
            endelse
            ;endif

        endif
    endif

    roisize = string(format='(f6.1,a4,f6.1)',(imageHeader.Wid * pixsize),' x ',(imageHeader.Len *pixsize))
    location = '(' + strtrim(imageHeader.sx,2) + ',' +  strtrim(imageHeader.sy,2)+') : ('+ strtrim(imageHeader.ex,2)+',' + strtrim(imageHeader.ey,2) + ')'
    stats = moment(img);   img is the square or rectangle roi.  we should calculate the mean brightness inside teh paricle boundary.
    ;***********************************************************
    sxsy=size(img,/dim)
    dx  =sxsy(0)
    dy  =sxsy(1)

    ; TAF 2014-03-28 putting in detection of when the .obj file is not present so that the raw pixel values can still be shown
    ; TAF 2014-08-19 this "if(ptr_valid(objData))" seems to be a bad way to detect if the .obj file is there -- for some reason
    ; it is returning zero even if the objData is valid, so the program is no longer displaying the mean, min/max, etc.
    ; TAF 2014-08-20 BUT, without this if statement, you can't get pixel location and value from the raw .roi file, you need a .obj file
    ;                so something needs to be done here to work with .roi files with and without .obj files,
    ;                since it is ridiculous not to have pixel location and value, etc always available!
    ;                Somehow with this if statement, we get pixel location and value, but never the mean/min/max displayed.  Without
    ;                this if statement, we get mean/min/max on files with .obj, but nothing for files without.  Confused as to why right now...

    imgmean = strtrim(stats[0],2) ; TAF 2014-08-21 using full ROI image for mean, this is replaced if the obj file exists
    ; and the boundary points can be used instead (see below Get_mean_brightness_inside_boundary)

    dev = strtrim(stats[1],2)
    minval = fix(min(img))
    maxval = fix(max(img))
    minval = strtrim(string(minval),2)+'/'+strtrim(string(maxval),2)
    maxval = strtrim(string(maxval),2)
    frameheader = (*(*pStateInfo).pFrameData).frameheader
    info = string(format='( Z0, Z5, F9.1, F9.1)',(imageHeader.Flags AND 'FFFF'X), imageHeader.Dark, imageHeader.Area, imageHeader.Perim)


    if(ptr_valid(objData)) then begin
        boundary=(*objData.pFrame)[index].imageheader.boundary
        boundaryPtsX=(*boundary) mod dx
        boundaryPtsY=(*boundary)/dx
        BoundaryPtsX=[BoundaryPtsX, boundaryPtsX(0)];  close the loop.
        BoundaryPtsy=[BoundaryPtsy, boundaryPtsy(0)];  close the loop; these is the polygon of the image.

        mean_brightness=Get_mean_brightness_inside_boundary(img,BoundaryPtsX, boundaryPTsY)

        if mean_brightness lt 0.0 then   stop

        mean_old = strtrim(stats[0],2)
        imgmean = strtrim(string(mean_brightness),2)
        ;            dev = strtrim(stats[1],2)
        ;            minval = fix(min(img))
        ;            maxval = fix(max(img))
        ;            minval = strtrim(string(minval),2)+'/'+strtrim(string(maxval),2)
        ;            maxval = strtrim(string(maxval),2)
        ;            frameheader = (*(*pStateInfo).pFrameData).frameheader
        ;            info = string(format='( Z0, Z5, F9.1, F9.1)',(imageHeader.Flags AND 'FFFF'X), imageHeader.Dark, imageHeader.Area, imageHeader.Perim)

        ;  in here I could input the mean_brightness within the boudary points.
        ;  print out the hole count, hole area and mean_old
        ;TAF commented out uneeded printouts from mouseover
        ;print, 'hole_count=',(*objData.pFrame)[index].imageheader.hole_count
        ;print, 'hole1_area=',(*objData.pFrame)[index].imageheader.hole1_area
        ;print, 'hole2_area=',(*objData.pFrame)[index].imageheader.hole2_area
        ;print, 'hole3_area=',(*objData.pFrame)[index].imageheader.hole3_area
        ;print, 'Mean_Old=', mean_old
        ;print, 'mean_inside_boundary=',mean_Brightness
        ;print,' mean_old - mean_inside=',mean_old -mean_Brightness
    endif
endif else begin
    if((*pStateInfo).autoextract)then if(not (*pStateInfo).Settings.Mosaic)then polyfill,[.02,.25,.25,.02,.02],[.40,.40,.5,.5,.40],/normal, color=0
    focus = '' ;0
    crystal = '' ;0
    cutoff = '' ;0
    reject = ''
    roisize = ''
    location = ''
    imgmean = ''
    mean_old=' '
    mean_brightness=' '
    dev = ''
    minval = '' ;0.0
    maxval = '' ;0.0
    info =''
    area = ''
    perimeter = ''
    roundness = ''
    length = ''
    width = ''
endelse


widget_control,(*pStateInfo).infoID.ROIsizeID,SET_VALUE = roisize
widget_control,(*pStateInfo).infoID.minID,SET_VALUE = minval
widget_control,(*pStateInfo).infoID.meanID,SET_VALUE = imgmean ;stats[0]
widget_control,(*pStateInfo).infoID.cutOffID,SET_VALUE = cutoff
widget_control,(*pStateInfo).infoID.HabitID,SET_VALUE = crystal
;widget_control,(*pStateInfo).infoID.LocationID,SET_VALUE = location
widget_control,(*pStateInfo).infoID.maxID,SET_VALUE = maxval
widget_control,(*pStateInfo).infoID.sigmaID,SET_VALUE = dev ;stats[1]
widget_control,(*pStateInfo).infoID.focusID,SET_VALUE = focus
widget_control,(*pStateInfo).infoID.areaID,SET_VALUE = area
widget_control,(*pStateInfo).infoID.perimeterID,SET_VALUE = perimeter
widget_control,(*pStateInfo).infoID.roundnessID,SET_VALUE = roundness
widget_control,(*pStateInfo).infoID.rejectID,SET_VALUE = reject
widget_control,(*pStateInfo).infoID.lengthID,SET_VALUE = length
widget_control,(*pStateInfo).infoID.widthID,SET_VALUE = width
;widget_control,(*pStateInfo).infoID.h2dsID,SET_VALUE = H2dsID;   added  2011-11-15
;widget_control,(*pStateInfo).infoID.v2dsID,SET_VALUE = V2DSID;   added  2011-11-15
widget_control,(*pStateInfo).infoID.h2dsID,SET_VALUE = H2dsID;   added  2011-11-15
widget_control,(*pStateInfo).infoID.v2dsID,SET_VALUE = V2DSID;   added  2011-11-15

;widget_control,(*pStateInfo).infoID.infoID,SET_VALUE = info
; boundary=(*objData.pFrame)[index].imageheader.boundary; is a point that hold the boudary index on the images.
; the boudary_1d should be replace with xnn ynn in teh processParticle routine.


end


;===============================================================================
FUNCTION GetROI,pStateInfo, x, y

    ; purpose:  this routine determine if the mouse is over a particle, and if so which one
    ;   PARAMETERs
    ;     pStateInfo:  main info structure
    ;     x:      x location of mouse in pixel coord
    ;     y:      y location of mouse in pixel coord

    ;   return
    ;     if over a particle , return index of particle and offset into particle in pixel-coords , else [-1,-1,-1]

    ; How many ROIs?
    n_imgs = n_elements(  *(*(*pStateInfo).pFrameData).pFrame )

    bounds = *(*pStateInfo).pBounds

    FOR i=0, n_imgs - 1 DO begin
        x0 = bounds[i,0]
        y0 = bounds[i,1]
        x1 = bounds[i,2]
        y1 = bounds[i,3]
        if ( x gt x0-1 and x lt x1+1 and y gt y0-1 and y lt y1+1 )then return,[i,x - x0,y - y0]
    endFOR

    ; Not inside any ROI
    return, [-1,-1,-1]
end


;===============================================================================
Function DoXSection, pStateInfo, index, XS_2, Xarr_2
    ;Purpose
    ;  to plot a cross sectional cut through the image

    ; Parameters
    ;  pStateInfo:  Input    state structure
    ;  index:       Input    index of ROI
    ;  XS_2:        Input    location of endpoint click in window co-ords
    ;  Xarr_2       Input    location of endpoint in array co-ords

    ; ROI is a structure of info for the particle in the frame
    ;ROI = (*(*(*pStateInfo).pFrameData).pFrame)[index]

    if((*pStateInfo).DispMode gt 0)then begin
        ; get settings from gui
        gradient = ((*pStateInfo).DispMode and 2)/2
        img = Disp_filt(*((*(*(*pStateInfo).pFrameData).pFrame)[index] ).pROI, gradient=gradient)
    endif else img = *((*(*(*pStateInfo).pFrameData).pFrame)[index] ).pROI

    img = rotate(img,7)
    zoom = (*pStateInfo).zoom

    ; get aspect ratio from window
    widget_control,(*pStateInfo).AspectID,get_value = YXAspect

    x1 = floor((*pStateInfo).Xarr_1[0] /(zoom*YXAspect))
    y1 = floor((*pStateInfo).Xarr_1[1] / zoom)

    x2 = floor(Xarr_2[0] /(zoom*YXAspect))
    y2 = floor(Xarr_2[1] / zoom)

    ;print,'(x1,y1)=',x1,y1,' (x2,y2)= ',x2,y2
    ; use XOR mode
    device, set_graphics_function=6

    ; erase old line if it exists
    if((*pStateInfo).XS_coords[0] ne -1 and not (*pStateInfo).redrawn)then begin
        x = [(*pStateInfo).XS_coords[0],(*pStateInfo).XS_coords[1]]
        y = [(*pStateInfo).XS_coords[2],(*pStateInfo).XS_coords[3]]
        plots,x,y,/device,thick = 4
    endif else (*pStateInfo).redrawn = !false

    ; draw a line on the display
    x0 = [(*pStateInfo).XS_1[0],XS_2[0]]
    y0 = [(*pStateInfo).XS_1[1],XS_2[1]]
    plots,x0,y0,/device,thick = 2

    ; return to copy mode
    device, set_graphics_function=3

    dx = x2-x1
    dy = y2-y1

    line_length_pixels  = (float(dx)^2. + float(dy)^2.)^.5
    line_length_pixels_string  = strtrim(string(format='(%"%7.1f")', line_length_pixels),  2)
    line_length_microns = line_length_pixels * (*pStateInfo).Settings.PixSize
    line_length_microns_string = strtrim(string(format='(%"%7.1f")', line_length_microns), 2)

    norm = (abs(dx) gt abs(dy))? float(abs(dx)):float(abs(dy))
    distance = findgen(norm)*  (sqrt(dx^2 + dy^2)/norm)



    dx = dx / norm
    dy = dy / norm
    if(dx ne 0)then x = indgen(norm)*dx    + x1[0] else x = intarr(norm) + x1[0]
    if(dy ne 0)then y = indgen(norm)*dy + y1[0] else y = intarr(norm) + y1[0]
    x = round(x)
    y = round(y)
    ;print,x1,x2,y1,y2
    ;print,'x',x
    ;print,'y',y

    ; put up a window, or set focus
    ;device,window_state=windowIDs ; check to see if the window was closed by the user
    ;valid = ((*pStateInfo).ProfID ne -1)? windowIDs[(*pStateInfo).ProfID]:0
    ;if((*pStateInfo).ProfID eq -1 or not valid)then begin
    ;  device,get_screen_size = dim
    ;  window,/free,xsize = 300,ysize = 300,xpos = dim[0]/2,ypos = 0,title='Profile Window'
    ;  (*pStateInfo).ProfID = !d.window
    ;endif else wset,(*pStateInfo).ProfID

    ; plot profile
    ;plot, dist, img[x,y],xtitle = 'Distance in Pixels along Line',ytitle='Particle Value',title='Cross Section Profile'
    ;oplot, dist, img[x,y],psym = 2,color = 125

    ; bring to front
    ;wshow,(*pStateInfo).ProfID

    ; return focus to main draw window
    ;wset,(*pStateInfo).MViewID

    set_plot,'z'
    device,set_resolution=[300,300]
    plot,  distance, img[x,y], xtitle = 'Pixel Distance (' + line_length_pixels_string + 'px, ' + line_length_microns_string + 'um)', $
        ytitle ='Particle Value', title = 'Cross Section Profile '
    oplot, distance, img[x,y], color = 255;psym = 2

    ;stop

    xsec = tvrd()
    set_plot,'win'
    tv,xsec,400,170
    ;wset,2
    ;plot,img[*,60]
    return,[x0,y0]
end

;===============================================================================
pro set_gui,pStateInfo

    ;set up some inital parameters based on the type of file
    case 1 of
        ((*pStateInfo).FileHead.DX EQ 464) AND ((*pStateInfo).FileHead.DY EQ 430 OR (*pStateInfo).FileHead.DY EQ 445):begin
        ;SPECIAL CASE FILE  FRIZZLE files
        (*pStateInfo).Settings.YXAspect = 1.0
        (*pStateInfo).Settings.PixSize = 1.0
        (*pStateInfo).FileHead.Ver = 17
    end
    ((*pStateInfo).FileHead.Ver LT 20 AND (*pStateInfo).FileHead.DX LT 1000):begin
    ;SPECIAL CASE FILE  SUCCESS files
    (*pStateInfo).Settings.YXAspect = 0.701
    (*pStateInfo).Settings.PixSize = 5.6
end
else:begin
;NORMAL FILE CPI files
(*pStateInfo).Settings.YXAspect = 1.0
(*pStateInfo).Settings.PixSize = !CPI_MICRONS_PER_PIXEL
end
endcase

; setup initial controls
WIDGET_CONTROL, (*pStateInfo).AspectID, SET_VALUE=1.0
WIDGET_CONTROL, (*pStateInfo).PixSizeID, SET_VALUE=!CPI_MICRONS_PER_PIXEL
WIDGET_CONTROL, (*pStateInfo).MosaicID, set_button=0
(*pStateInfo).Settings.Mosaic = 0

; start at a zoom of 1
(*pStateInfo).zoom = 1.0
widget_control,(*pStateInfo).zoomLabelID,set_value = strtrim((*pStateInfo).zoom,2)
widget_control,(*pStateInfo).zoomsliderID,set_value = 10

; new file so set for frame 1
(*pStateInfo).ImgPos = 1
; ImgType codes
;  -1   HouseKeeping Frame
;  0    ERROR
;  1    normal roi frame
;  2    background
;(*pStateInfo).pImgPosindx = ptr_new(where( (*(*pStateInfo).pFileScan).imgtype eq 1,ImgCount ))
(*pStateInfo).pImgPosindx = ptr_new(where( (*(*pStateInfo).pFileScan).imgtype gt 0,ImgCount ))
FirstImgPos = 0
(*pStateInfo).LastImgPos = MAX((*(*pStateInfo).pImgPosindx), MIN=FirstImgPos)
(*pStateInfo).FirstImgPos = FirstImgPos
if(ImgCount eq 0) then begin
    ShowMsg, (*pStateInfo).InstLabel, 'File Does Not Contain ROI Images.'
    message,'Error. no images in this file',/error
    return
endif

; set range for frame slider
imgCnt = n_elements(*(*pStateInfo).pImgPosindx)
WIDGET_CONTROL, (*pStateInfo).ImgSliderID, SET_VALUE=1
WIDGET_CONTROL, (*pStateInfo).ImgSliderID, SET_SLIDER_MIN=1
WIDGET_CONTROL, (*pStateInfo).ImgSliderID, SET_SLIDER_MAX=imgCnt
(*pStateInfo).TotImgCount = imgCnt

set_TLB_title,pStateInfo

end

