; code from aaron Feb. 7, 2008
; Contains: forback, min_circle2, read_ab, write_ab, condition_image, abdisp.pro, abdisp2.pro

FUNCTION ABparams2, img, dp=dp, res=res, smooth_boundary=smooth_boundary, contourlevel=contourlevel, debug=debug
    ;FUNCTION to find the perimeter, size, and focus of a CPI image.
    ;Inputs:
    ;img:  The image to process.  This may or may not have the background removed.
    ;dp:   The distance between each point on the perimeter (in same units as res).
    ;res:  The resolution of the image (microns/pixel)
    ;smooth_boundary:  [1,0], turns on/off a light boundary smoothing filter
    ;contourlevel:     The level to contour, as a fraction of the background intensity
    ;Aaron Bansemer, NCAR, Jan 2008
    forward_function foreback,min_circle2

    ;-----------Default values-----------------------------
    IF n_elements(dp)  eq 0 THEN dp  = !CPI_MICRONS_PER_PIXEL
    IF n_elements(res) eq 0 THEN res = !CPI_MICRONS_PER_PIXEL ;default resolution
    IF n_elements(smooth_boundary) eq 0 THEN smooth_boundary=0
    IF n_elements(contourlevel) eq 0 THEN contourlevel=0.6 ;default light/dark threshold to contour
    IF dp lt res THEN stop,'dp cannot be less than res in perimeter.pro'

    idist=float(dp)/res     ;dp in pixel units, will be used throughout this program
    maxpoints=5000

    ;----------Make sure particle is OK--------------------
    nullstructure={x:0.0,y:0.0,perimeter:0.0,focus:0.0,thresh:0.0,diam:0.0,diam_corrected:0.0,$
        xcentroid:0.0,ycentroid:0.0,fitcenter:[0.0,0.0],area:0.0,error:1}
    s=size(img,/dim)
    IF min(s) lt 2.0*idist THEN return,nullstructure  ;This particle is too small to work
    xmax=s[0]-1
    ymax=s[1]-1

    ;----------Filter image, find threshold and start point----
    fimg=float(filterimg(img))    ;Use the filtered image to do the perimeter finding.
    levels=foreback(img)          ;Find foreground and background values
    IF levels.back-min(fimg) lt 20 THEN return,nullstructure    ;This is probably wallpaper
    thresh=min(fimg)+(levels.back-min(fimg)) * contourlevel     ;This will be the level to 'contour'

    ;Start halfway down on the left side and look across until the first point is reached
    y=s[1]/2
    w=where(fimg[*,y] lt thresh,nw)

    ;Didn't find a point? Try 1/4 of the way down instead
    IF nw eq 0 THEN BEGIN
        y=s[1]/4
        w=where(fimg[*,y] lt thresh,nw)
    ENDIF

    ;Last try, 3/4 of the way down
    IF nw eq 0 THEN BEGIN
        y=3*s[1]/4
        w=where(fimg[*,y] lt thresh,nw)
    ENDIF

    IF nw eq 0 THEN return,nullstructure ;Cannot find a starting point

    ;This will be our starting point (not used as a perimeter point)
    x=(w[0]-0.5)>0   ;-0.5 since it sometimes goes a tad too far

    ;-----Begin the search around the particle------
    xp=fltarr(maxpoints)  ;x and y coordinates of the perimeter points
    yp=fltarr(maxpoints)
    xp[0]=x
    yp[0]=y
    i=0
    test_angle=3*!pi/2
    num_angles=((10*idist)<90)
    angle_res=2*!pi/num_angles
    t=levels.back
    REPEAT BEGIN
        i=i+1
        j=0
        reach=1.0

        ;Create the search circle
        REPEAT BEGIN
            lastt=t
            xtest=xp[i-1]+idist*sin(test_angle)*reach
            ytest=yp[i-1]+idist*cos(test_angle)*reach

            ;Tried doing these all at once, and binary searching.  Minimal speed improvement.
            t=interpolate(fimg,xtest,ytest)

            ;Test for out of bounds points
            IF (xtest gt xmax) or (xtest lt 0) or (ytest gt ymax) or (ytest lt 0) THEN t=thresh+0.01
            test_angle=test_angle+angle_res

            ;Rarely no crossings are found.  Temporarily extend the reach for those.
            IF j gt num_angles THEN BEGIN
                reach=reach+1
                j=0
            ENDIF
            IF reach gt 5 THEN return,nullstructure
            j=j+1
        ENDREP UNTIL (t lt thresh) and (lastt gt thresh)

        angle=(test_angle-angle_res)-(thresh-t)/(lastt-t)*angle_res
        xp[i]=xp[i-1]+idist*sin(angle) > 0 < xmax
        yp[i]=yp[i-1]+idist*cos(angle) > 0 < ymax
        test_angle=angle-!pi/2


        IF keyword_set(debug) THEN BEGIN
            plot,xp[0:i],yp[0:i],xr=[0,xmax],yr=[0,ymax],/xs,/ys,/iso,psym=-2
            wait,0.1
            ;stop
        ENDIF
    ENDREP UNTIL (i gt 5 and sqrt((xp[i]-xp[1])^2+(yp[i]-yp[1])^2) lt idist) or i eq maxpoints-1
    IF i eq maxpoints-1 THEN return, nullstructure  ;Probably got caught in a loop

    x=xp[1:i]
    y=yp[1:i]

    ;--------------Compute the focus---------------------
    normimg=(img*200.0/levels.back) < 255 ;Sobel filter, normalized to background of 200
    s=sobel(normimg)

    xr=round(x) ;Use the rounded, unsmoothed, indices for better results
    yr=round(y)
    focusp=fltarr(i)
    goodpoints=bytarr(i)+1

    FOR j=0,i-1 DO BEGIN
        ;Don't use points on the edges
        IF (xr[j] eq 0) or (xr[j] eq xmax) or (yr[j] eq 0) or (yr[j] eq ymax) THEN goodpoints[j]=0
        focusp[j]=mean(s[(xr[j]-3)>0:(xr[j]+3)<xmax, (yr[j]-3)>0:(yr[j]+3)<ymax])
    ENDFOR

    w=where(goodpoints eq 1,nw)
    IF nw gt 0 THEN focus=mean(focusp[w])/3.0 ELSE focus=0

    ;-------------Smooth boundary-----------------------
    IF smooth_boundary THEN BEGIN
        ;Just doing a cheap edge wrap
        tempx=smooth([x,x],3)
        tempy=smooth([y,y],3)
        x=tempx[3:i+2]
        y=tempy[3:i+2]
    ENDIF

    ;-----------Compute final values--------------------
    circlefit=min_circle2(x,y)
    diam=circlefit.diam*res
    diam_corrected=diam-(100-(focus<100))/4.0
    perimeter=total(sqrt((x-shift(x,1))^2+(y-shift(y,1))^2))*res

    ;Area and centroid in pixel coordinates
    area=total((x*shift(y,1)-y*shift(x,1)))/2.0   ;Gauss 1795
    xcentroid=total((x+shift(x,1))*(x*shift(y,1)-y*shift(x,1)))/(6*area)
    ycentroid=total((y+shift(y,1))*(x*shift(y,1)-y*shift(x,1)))/(6*area)
    area=area*res^2 ;Convert area to microns

    return,{x:x, y:y, perimeter:perimeter, focus:focus, thresh:thresh, diam:diam,$
        diam_corrected:diam_corrected, area:area, fitcenter:circlefit.center,$
        xcentroid:xcentroid, ycentroid:ycentroid, ngood:nw, error:0}
END; end of abparams2 function


FUNCTION condition_image, roi, background
    ;FUNCTION to condition an roi for optimal processing.
    ;Removes background only in object area.
    ;AB 1/07
    forward_function foreback
    roi2=roi
    a=abs(fix(background)-roi)
    w=where(a lt 50)
    IF w[0] ne -1 THEN roi2[w]=roi[w]-background[w]+(foreback(background)).back
    IF (min(roi2) lt 0) or (max(roi2) gt 255) then stop,'out of range in condition_image'

    return,roi2
END;end of condition_images


PRO abdisp2,fn,zoom=zoom
    IF n_elements(zoom) eq 0 THEN zoom=1
    IF n_elements(smooth_boundary) eq 0 THEN smooth_boundary=0

    keystroke='1'

    loadct,0
    tvlct,255,0,0,255
    tvlct,0,255,255,254

    close,1
    openr,1,fn

    k=read_ab(1)
    thisframe=k.frame

    REPEAT BEGIN

        c=0
        xx=0
        yy=0
        top=0

        REPEAT k=read_ab(1) UNTIL k.frame ge thisframe+fix(keystroke)
        thisframe=k.frame
        window,1,xsize=1100,ysize=1100
        xyouts,500,1000,string(sfm2hms(k.time)),/dev
        REPEAT BEGIN
            c=c+1

            s=size(k.image)
            IF xx+s[1] gt 1200 THEN BEGIN
                xx=0
                yy=yy+top+5
                top=0
            ENDIF

            img=k.image<253
            img[k.cbound]=254
            img[k.ax,k.ay]=255

            IF zoom ne 1 THEN tv,congrid(img,s[1]*zoom,s[2]*zoom),xx,yy ELSE tv,img,k.sx,k.sy
            xyouts,k.sx-20,k.sy+5,strtrim(string(fix(k.alen)),2),color=255,/dev
            xyouts,k.sx-20,k.sy+15,strtrim(string(fix(k.clen)),2),color=254,/dev
            xyouts,k.sx+s[1]+2,k.sy+5,strtrim(string(fix(k.afocus)),2),color=255,/dev
            xyouts,k.sx+s[1]+2,k.sy+15,strtrim(string(fix(k.cfocus)),2),color=254,/dev
            IF fix(k.alen) eq 59 and fix(k.clen) eq 68 THEN print,k.sx,k.sy
            xx=xx+s[1]*zoom+5
            top=top>(s[2]*zoom+25)
            k=read_ab(1)

        ENDREP UNTIL k.frame ne thisframe or eof(1)
        ;thisframe=k.frame
        oldkeystroke=keystroke

        read,keystroke
        IF keystroke eq '' THEN keystroke=oldkeystroke
        IF keystroke eq 'q' THEN stop
    ENDREP UNTIL eof(1) ;or keystroke eq 'q'
    close,1
    ;stop
END  ; end of abdisp2.pro


PRO abdisp,fn,zoom=zoom
    IF n_elements(zoom) eq 0 THEN zoom=1
    IF n_elements(smooth_boundary) eq 0 THEN smooth_boundary=0

    keystroke='50'

    loadct,0
    tvlct,255,0,0,255
    tvlct,0,255,255,254

    close,1
    openr,1,fn

    REPEAT BEGIN
        c=0
        xx=0
        yy=0
        top=0
        window,1,xsize=1200,ysize=1200
        REPEAT BEGIN
            c=c+1
            k=read_ab(1)
            s=size(k.image)
            IF xx+s[1] gt 1200 THEN BEGIN
                xx=0
                yy=yy+top+5
                top=0
            ENDIF

            img=k.image<253
            img[k.cbound]=254
            img[k.ax,k.ay]=255

            IF zoom ne 1 THEN tv,congrid(img,s[1]*zoom,s[2]*zoom),xx,yy ELSE tv,img,xx,yy
            xyouts,xx+5,yy+5,strtrim(string(fix(k.alen)),2),color=255,/dev
            xyouts,xx+5,yy+15,strtrim(string(fix(k.clen)),2),color=254,/dev
            xyouts,xx+s[1]*zoom-20,yy+5,strtrim(string(fix(k.afocus)),2),color=255,/dev
            xyouts,xx+s[1]*zoom-20,yy+15,strtrim(string(fix(k.cfocus)),2),color=254,/dev

            xx=xx+s[1]*zoom+5
            top=top>(s[2]*zoom+25)


        ENDREP UNTIL c eq fix(keystroke) or eof(1)
        oldkeystroke=keystroke
        read,keystroke
        IF keystroke eq '' THEN keystroke=oldkeystroke
        IF keystroke eq 'q' THEN stop
    ENDREP UNTIL eof(1) ;or keystroke eq 'q'
    close,1
    ;stop


    ;   IF zoom ne 1 THEN tv,congrid(img,s[1]*zoom,s[2]*zoom,/interp),xx,yy ELSE tv,img,xx,yy
    ;   xyouts,xx,yy+s[2]*zoom+5,'p='+strtrim(string(round(a.perimeter)),2),/device
    ;   ;xyouts,xx,yy+s[2]*zoom+15,'f='+strtrim(string(round(f[4])),2),/device
    ;   xyouts,xx,yy+s[2]*zoom+15,'f2='+strtrim(string(round(a.focus)),2),/device
    ;   xyouts,xx,yy+s[2]*zoom+25,'d='+strtrim(string(round(a.diam)),2),/device
    ;   focus[n]=a.focus
    ;   ;print,a.diam
    ;   xx=xx+s[1]*zoom+5
    ;   top=top>(s[2]*zoom+25)

    ;ENDFOR



    ;Diplay again in order of increasing focus
    ;isort=sort(focus)
    ;xx=0
    ;yy=yy+top+50
    ;top=0
    ;FOR i=0,n_elements(roi)-1 DO BEGIN
    ;   img=*roi[isort[i]].proi
    ;   s=size(img)
    ;   IF xx+s[1] gt 1200 THEN BEGIN
    ;      xx=0
    ;      yy=yy+top+5
    ;      top=0
    ;   ENDIF
    ;   IF zoom ne 1 THEN tv,congrid(img,s[1]*zoom,s[2]*zoom,/interp),xx,yy ELSE tv,img,xx,yy
    ;   xx=xx+s[1]*zoom+5
    ;   top=top>s[2]*zoom

    ;ENDFOR




    ;stop
END  ; end of abdisp.pro


function min_circle2,x,y
    ;***NOTE:  This is only an approximation!! *******
    ;Usually about 5% too big

    ;finds the minimum bounding circle for a set of points
    ;given by x and y.
    ;Based on algorithm in Graphics Gems I, article by Ritter
    ;Code translated from C, found on www.acm.org/pubs/tog/GraphicsGems/category.html
    ;Aaron Bansemer, 4-2003

    area=n_elements(x)
    if area eq 1 then return,{diam:0.0,center:[1,1]}
    if area eq 2 then return,{diam:1.2,center:[1,1]} ;average value of 2 pixes in a row or diagonally
    x=float(x)
    y=float(y)

    ;FIRST PASS - get minima and maxima
    ymin=min(y,ymini)
    xmin=min(x,xmini)
    ymax=max(y,ymaxi)
    xmax=max(x,xmaxi)

    ;Set xspan = distance between the 2 points xmin & xmax (squared) *
    dx=x[xmaxi]-x[xmini]
    dy=y[xmaxi]-y[xmini]
    xspan=dx^2 + dy^2

    dx=x[ymaxi]-x[ymini]
    dy=y[ymaxi]-y[ymini]
    yspan=dx^2 + dy^2

    ;Set points dia1 & dia2 to the maximally separated pair
    dia1 = xmini & dia2 = xmaxi  ;assume xspan biggest
    maxspan = xspan
    IF yspan gt maxspan THEN BEGIN
        maxspan=yspan
        dia1=ymini & dia2=ymaxi
    ENDIF

    ;dia1,dia2 is a diameter of initial sphere
    ;calc initial center
    cenx=(x[dia1]+x[dia2])/2
    ceny=(y[dia1]+y[dia2])/2

    ;calculate initial radius
    dx=x[dia2]-cenx
    dy=y[dia2]-ceny
    rad_sq=dx^2 + dy^2
    rad=sqrt(rad_sq)

    ;print,cenx,ceny,rad*2
    ;SECOND PASS: increment current disk
    FOR i=0,area-1 DO BEGIN
        dx=x[i]-cenx
        dy=y[i]-ceny
        old_to_p_sq= dx^2 +dy^2
        IF old_to_p_sq gt rad_sq THEN BEGIN
            ;point is outside current disk
            old_to_p = sqrt(old_to_p_sq)
            ;calc radius of new sphere
            rad = (rad + old_to_p) / 2.0
            rad_sq = rad*rad       ;for next r**2 compare
            old_to_new = old_to_p - rad
            ;calc center of new sphere
            cenx = (rad*cenx + old_to_new*x[i]) / old_to_p
            ceny = (rad*ceny + old_to_new*y[i]) / old_to_p
        ENDIF
    ENDFOR
    return,{diam:rad*2, center:[cenx,ceny]}  ;RETURNS *DIAMETER*
END ; end of min_circle2.pro


FUNCTION foreback,img,binsize=binsize
    ;FUNCTION to find the background and foreground levels of an image
    ;requires a gray level CPI image ranging from 0 to 255
    ;Assumed mean image value is between the foreground and background... should
    ;be valid for most in-focus particles.
    ;AB 1/07

    IF n_elements(binsize) eq 0 THEN binsize=3 ;to ignore some noise
    z=smooth(histogram(img,binsize=binsize,max=255,min=0),5,/edge_truncate)

    i=fix(mean(img)/binsize)>1; mo added >1
    dummy=max(z[0:i-1],idark)
    dummy=max(z[i:*],ibright)
    ibright=ibright+i

    return,{back:ibright*binsize,fore:idark*binsize}
END; end offoreback.pro

FUNCTION read_ab, lun
    ;FUNCTION to read in the next roi from an "ab" file
    ;AB 1/08

    h={sync:0L, xsize:0L, ysize:0L, nab:0L, ncv:0L}
    readu,1,h

    data={frame:0L,$
        time:0L,$
        image:bytarr(h.xsize,h.ysize),$   ;byte
        sx:0US,$      ;uint
        sy:0US,$      ;uint
        ex:0US,$      ;uint
        ey:0US,$      ;uint
        clen:0.0,$       ;float
        cwid:0.0,$       ;float
        carea:0.0,$      ;float
        cperim:0.0,$     ;float
        cround:0.0,$     ;float
        ccentroid:fltarr(2),$    ;float(2)
        cbound:lonarr(h.ncv),$   ;long array
        cfocus:0B,$              ;byte

        alen:0.0,$        ;float
        aperim:0.0,$
        afocus:0.0,$
        acentroid:fltarr(2),$ ;float(2)
        ax:fltarr(h.nab),$             ;float
        ay:fltarr(h.nab)}

    readu,lun,data
    return,data
END;  end of read_ab


PRO write_ab, lun, image, background, cv, frame, time
    ;Write the abparams structure to a raw data file. mo added---new-obj data file.
    ;AB 1/08

    s=size(image,/dim)
    ;Check to make sure this is a valid particle
    IF s[0] eq 1024 or s[1] eq 1024 THEN return

    ;Make sure the cpiview processing is valid
    IF size(cv,/type) ne 8 THEN return  ;must be a structure


    cimage=condition_image(image, background);    *pROIImg - fix(backROI) + (foreback(backroi)).back  ;ab
    ;stop
    ;cimage=image
    ab= abparams2(cimage,dp=5,contour=0.6);  ;mo add: this finds the parameters, new size ect of the
    ; images, replace the old Len and width ect.
    data={sync:'55555555'x,$
        s:long(s),$
        nab:long(n_elements(ab.x)),$
        ncv:long(n_elements(*(cv.boundary)[0])),$
        frame:long(frame),$
        time:long(time),$
        image:image,$   ;byte
        sx:cv.sx,$      ;uint
        sy:cv.sy,$      ;uint
        ex:cv.ex,$      ;uint
        ey:cv.ey,$      ;uint
        clen:cv.len    * !CPI_MICRONS_PER_PIXEL, $                   ; float
        cwid:cv.wid    * !CPI_MICRONS_PER_PIXEL, $                   ; float
        carea:cv.area  * !CPI_MICRONS_PER_PIXEL * !CPI_MICRONS_PER_PIXEL, $ ; float
        cperim:cv.perim* !CPI_MICRONS_PER_PIXEL, $                   ; float
        cround:cv.roundness1,$   ;float
        ccentroid:cv.centroid,$ ;float(2)
        cbound:*(cv.boundary)[0],$   ;long array
        cfocus:cv.focus,$       ;byte

        alen:ab.diam,$        ;float  ; mo added: this is aaron's new parameters. new length,
        aperim:ab.perimeter,$         ; new perimeter
        afocus:ab.focus,$             ; new focus values.
        acentroid:ab.center,$ ;float(2) ; new centroid of the particle.
        ax:ab.x,$             ;float array  ; new x and new y.
        ay:ab.y}

    writeu,lun,data
END; end of write_ab
FUNCTION ABparams2_mo_modified, img, dp=dp, res=res, smooth_boundary=smooth_boundary, contourlevel=contourlevel, debug=debug
    ;FUNCTION to find the perimeter, size, and focus of a CPI image.
    ;Inputs:
    ;img:  The image to process.  This may or may not have the background removed.
    ;dp:   The distance between each point on the perimeter (in same units as res).
    ;res:  The resolution of the image (microns/pixel)
    ;smooth_boundary:  [1,0], turns on/off a light boundary smoothing filter
    ;contourlevel:     The level to contour, as a fraction of the background intensity
    ;Aaron Bansemer, NCAR, Jan 2008


    ;-----------Default values-----------------------------
    IF n_elements(dp)  eq 0 THEN dp  = !CPI_MICRONS_PER_PIXEL
    IF n_elements(res) eq 0 THEN res = !CPI_MICRONS_PER_PIXEL ;default resolution
    IF n_elements(smooth_boundary) eq 0 THEN smooth_boundary=0
    IF n_elements(contourlevel) eq 0 THEN contourlevel=0.6 ;default light/dark threshold to contour
    IF dp lt res THEN stop,'dp cannot be less than res in perimeter.pro'

    idist=float(dp)/res     ;dp in pixel units, will be used throughout this program
    maxpoints=5000

    ;----------Make sure particle is OK--------------------
    nullstructure={x:0.0,y:0.0,perimeter:0.0,focus:0.0,diam:0.0,center:[0.0,0.0],area:0.0,$
        len:0.0,wid:0.0,centroid:[0.0,0.0],harmonics:0.0,rshape:0.0,error:1}
    s=size(img,/dim)
    IF min(s) lt 2.0*idist THEN return,nullstructure  ;This particle is too small to work
    xmax=s[0]-1
    ymax=s[1]-1

    ;----------Filter image, find threshold and start point----
    fimg=float(filterimg(img))    ;Use the filtered image to do the perimeter finding.
    levels=foreback(img)          ;Find foreground and background values
    thresh=min(fimg)+(levels.back-min(fimg)) * contourlevel  ;This will be the level to 'contour'

    ;Start halfway down on the left side and look across until the first point is reached
    y=s[1]/2
    w=where(fimg[*,y] lt thresh,nw)

    ;Didn't find a point? Try 1/4 of the way down instead
    IF nw eq 0 THEN BEGIN
        y=s[1]/4
        w=where(fimg[*,y] lt thresh,nw)
    ENDIF

    ;Last try, 3/4 of the way down
    IF nw eq 0 THEN BEGIN
        y=3*s[1]/4
        w=where(fimg[*,y] lt thresh,nw)
    ENDIF

    IF nw eq 0 THEN return,nullstructure ;Cannot find a starting point

    ;This will be our starting point (not used as a perimeter point)
    x=(w[0]-0.5)>0   ;-0.5 since it sometimes goes a tad too far

    ;-----Begin the search around the particle------
    xp=fltarr(maxpoints)  ;x and y coordinates of the perimeter points
    yp=fltarr(maxpoints)
    xp[0]=x
    yp[0]=y
    i=0
    test_angle=3*!pi/2
    num_angles=((10*idist)<90)
    angle_res=2*!pi/num_angles
    t=levels.back
    REPEAT BEGIN
        i=i+1
        j=0
        reach=1.0

        ;Create the search circle
        REPEAT BEGIN
            lastt=t
            xtest=xp[i-1]+idist*sin(test_angle)*reach
            ytest=yp[i-1]+idist*cos(test_angle)*reach

            ;Tried doing these all at once, and binary searching.  Minimal speed improvement.
            t=interpolate(fimg,xtest,ytest)

            ;Test for out of bounds points
            IF (xtest gt xmax) or (xtest lt 0) or (ytest gt ymax) or (ytest lt 0) THEN t=thresh+0.01
            test_angle=test_angle+angle_res

            ;Rarely no crossings are found.  Temporarily extend the reach for those.
            IF j gt num_angles THEN BEGIN
                reach=reach+1
                j=0
            ENDIF
            IF reach gt 5 THEN return,nullstructure
            j=j+1
        ENDREP UNTIL (t lt thresh) and (lastt gt thresh)

        angle=(test_angle-angle_res)-(thresh-t)/(lastt-t)*angle_res
        xp[i]=xp[i-1]+idist*sin(angle) > 0 < xmax
        yp[i]=yp[i-1]+idist*cos(angle) > 0 < ymax
        test_angle=angle-!pi/2


        IF keyword_set(debug) THEN BEGIN
            plot,xp[0:i],yp[0:i],xr=[0,xmax],yr=[0,ymax],/xs,/ys,/iso,psym=-2
            wait,0.1
            ;stop
        ENDIF
    ENDREP UNTIL (i gt 5 and sqrt((xp[i]-xp[1])^2+(yp[i]-yp[1])^2) lt idist) or i eq maxpoints-1
    IF i eq maxpoints-1 THEN return, nullstructure  ;Probably got caught in a loop

    x=xp[1:i]
    y=yp[1:i]

    ;--------------Compute the focus---------------------
    normimg=(img*200.0/levels.back) < 255 ;Sobel filter, normalized to background of 200
    s=sobel(normimg)

    xr=round(x) ;Use the rounded, unsmoothed, indices for better results
    yr=round(y)
    focusp=fltarr(i)
    goodpoints=bytarr(i)+1

    FOR j=0,i-1 DO BEGIN
        ;Don't use points on the edges
        IF (xr[j] eq 0) or (xr[j] eq xmax) or (yr[j] eq 0) or (yr[j] eq ymax) THEN goodpoints[j]=0
        focusp[j]=mean(s[(xr[j]-3)>0:(xr[j]+3)<xmax, (yr[j]-3)>0:(yr[j]+3)<ymax])
    ENDFOR

    w=where(goodpoints eq 1,nw)
    IF nw gt 0 THEN focus=mean(focusp[w])/3.0 ELSE focus=0

    ;-------------Smooth boundary-----------------------
    IF smooth_boundary THEN BEGIN
        ;Just doing a cheap edge wrap
        tempx=smooth([x,x],3)
        tempy=smooth([y,y],3)
        x=tempx[3:i+2]
        y=tempy[3:i+2]
    ENDIF

    ;-----------Compute final values--------------------
    circlefit=min_circle2(x,y)
    diam=circlefit.diam*res
    diam_corrected=diam-(100-(focus<100))/4.0
    perimeter=total(sqrt((x-shift(x,1))^2+(y-shift(y,1))^2))*res
    area=total((x*shift(y,1)-y*shift(x,1))[0:i-2])/2.0*res*res


    ;find the maximum length and width of the particle, and its centroid and rshape and the harmonics.
    Centroid=fltarr(2)
    centroid(0)=mean(x)
    centroid(1)=mean(y)

    ; search along this direction for the largest distance ( ie. the width) (see shapeLength)
    YXASPECT=1.0
    nnn=n_elements(x);  nnn must be greater than 2.  Must have at least 2 points.
    MaxDist = 1.0
    FOR i=0L, nnn-2 DO BEGIN
        ; these are the delta x and delta y from the ith point to all the others
        dx = long(X[i] - X[i+1:nnn-1])
        dy = long(Y[i] - Y[i+1:nnn-1])

        ; length from ith point to all others
        length = sqrt(FLOAT(dx*YXASPECT)^2 + FLOAT(dy)*dy)
        ;update MaxDist to be the longest so far
        MaxDist = max(length) > MaxDist

        indx = where(length EQ MaxDist, Matches)
        IF (Matches GT 0) THEN BEGIN
            ; here if there are more than 1 longest vectors; lp1 lp2 are the longest length two points.
            IF X[i] LT X[i+1+indX[0]] THEN BEGIN
                lp1 = i
                lp2 = i+1+indX[0]
            ENDIF ELSE BEGIN
                lp2 = i;  index of one point
                lp1 = i+1+indX[0]; index of the other point
            ENDELSE
        ENDIF
    ENDFOR
    len=Maxdist

    ; this is the perpendicular angle from longest dimension
    angle = atan(Y(lp2)-Y(lp1), X(lp2)-X(lp1))


    ; find the width; That is perdendicular to Length.
    IF angle LT 0 THEN angle = angle + !PI
    IF angle LT !PI/2 THEN angle = angle + !PI/2 $
        ELSE angle = angle - !PI/2

    MaxDist=1.0
    FOR i=0L, nnn-2 DO BEGIN
        dx = long(X[i+1:nnn-1] - X[i])
        dy = long(Y[i+1:nnn-1] - Y[i])
        AngleNow = atan(dy,dx)
        WIndx = where(AngleNow LT 0, Matches)
        IF (Matches GT 0) THEN AngleNow[WIndx] = AngleNow[WIndx] + !PI

        WIndx = where(abs(AngleNow - angle) LT 0.1, Matches)

        IF (Matches GT 0) THEN BEGIN
            wid = sqrt(FLOAT(YXASPECT*dx[WIndx])^2 + FLOAT(dy[WIndx])^2)
            MaxDist = max(wid) > MaxDist
            indx = where(wid EQ MaxDist, Matches)

            IF (Matches GT 0) THEN BEGIN
                IF X[i] LT X[i+1+WIndx[indx[0]]] THEN BEGIN
                    wp1 = i
                    wp2 = i+1+WIndx[indx[0]]
                ENDIF ELSE BEGIN
                    wp2 = i
                    wp1 = i+1+WIndx[indx[0]]
                ENDELSE
            ENDIF
        ENDIF
    ENDFOR
    Width=Maxdist
    ; calculate rshape ---radiis of the particle, it is a vector
    harmonics=ShapeHarmonics_mo(X, Y, Centroid, rShape=rShape)

    return,{x:x,y:y,perimeter:perimeter,focus:focus,diam:circlefit.diam*res,$
        diam_corrected:diam_corrected,center:circlefit.center,area:area,$
        len:len,wid:width,centroid:centroid,harmonics:harmonics,rshape:rshape,error:0}
END ; aaron's new abparams2. to renew heuse apints,, f
